/*
 * Programming in Java, Jacobs University, 2015
 *
 * This file implements a bounded buffer using explicit locks and
 * condition variables.
 */

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;
			
public class BoundedBuffer
{
    private final int size = 8;
    private int count = 0, out = 0, in = 0;
    private int[] buffer = new int[size];
    private Lock lock = new ReentrantLock();
    private Condition cond = lock.newCondition();
    
    public void insert(int i)
    {
	lock.lock();
	try {
	    while (count == size) {
		try {
		    cond.await();
		} catch (InterruptedException e) { }
	    }
	    buffer[in] = i;
	    in = (in + 1) % size;
	    count++;
	    cond.signalAll();    // wakeup all waiting threads
	} finally {
	    lock.unlock();
	}
    }
    
    public int remove()
    {
	int r;

	lock.lock();
	try {
	    while (count == 0) {
		try {
		    cond.await();
		} catch (InterruptedException e) { }
	    }
	    r = buffer[out];
	    out = (out + 1) % size;
	    count--;
	    cond.signalAll();    // wakeup all waiting threads
	} finally {
	    lock.unlock();
	}
	return r;
    }
}
